==> wmifinfo-0.02.tgz: wmifinfo-0.02/wmifinfo.c <==
	 
#include <stdio.h>
#include <unistd.h>
#include <getopt.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
#include <X11/X.h>
#include <X11/xpm.h>
#include <arpa/inet.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <net/if.h>
#include <errno.h>

#include "xutils.h"
#include "bitmaps/wmifinfo_led.xbm"
#include "bitmaps/wmifinfo_led.xpm"
#include "bitmaps/wmifinfo_lcd.xbm"
#include "bitmaps/wmifinfo_lcd.xpm"

#define MAXIFS 10
#define DELAY 100000L

char	LabelColor[30]    	= "#79bdbf";
char	WindGustColor[30] 	= "#ff0000";
char	DataColor[30]     	= "#ffbf50";
char	BackColor[30]     	= "#181818";
char	StationTimeColor[30]    = "#c5a6ff";

char startif[16] = "";
char ifname[MAXIFS][16];
int ifaces;
int ifno = 0;
struct ifinfo_t ifinfo;
int fd = 0;
struct ifconf ifc;

struct ifinfo_t {
	char id[16];
	int state;
	char hw[6];
	in_addr_t ip;
	in_addr_t nm;
	in_addr_t gw;
	int sl;
	int bytes;
};

struct font_t {
	char *chars;
	int sx;
	int sy;
	int dx;
	int dy;
	int charspline;
};


#define MODE_LED 1
#define MODE_LCD 2

int mode = MODE_LED;

void parse_cmdline(int argc, char *argv[]);
void ButtonPressEvent(XButtonEvent *);
char *strupper(char *str);
void getifnames(void);
int getifinfo(char *ifname, struct ifinfo_t *info);


struct font_t font1 = { " 0123456789ABCDEF", 64, 74, 4, 5, 17};
struct font_t font2 = { "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789: ", 1, 65, 5, 7, 26};
	

void drawtext(char *str, struct font_t *font, int x, int y)
{
	int i = 0;
	int ix, iy;
	char *p;
		
	while(str[i]) {
		p = strchr(font->chars, str[i]);
		ix = (p) ? (p - font->chars) : 0;
		
		iy = (ix / font->charspline);
		ix = (ix % font->charspline);
	
		copyXPMArea(	font->sx + ix * font->dx, 
				font->sy + iy * font->dy,
				font->dx, 
				font->dy,
				x + font->dx * i, 
				y);
				
		i++;
	}
}


void drawipaddr(in_addr_t addr, int linenum)
{
	char buf[4];
	int i;
	
	for(i=0; i<4; i++) {
		snprintf(buf, 4, "%3d", (addr >> (i*8)) & 255);
		drawtext(buf, &font1, 5 + i*14, 19 + linenum*9);
	}
}

void drawhwaddr(unsigned char *addr)
{
	char buf[4];
	int i;
	
	for(i=0; i<6; i++) {
		snprintf(buf, 4, "%02X", addr[i]);
		drawtext(buf, &font1, 6 + i*9, 46);
	}
}

int main(int argc, char *argv[])
{

	XEvent event;
	char buf[16];
	int d=0;
	int pifno=-1;
	int lastbytes=0;
	int x;
	
	parse_cmdline(argc, argv);

	initXwindow(argc, argv);

	if(mode == MODE_LED) {
		openXwindow(argc, argv, wmifinfo_led_xpm, wmifinfo_led_bits,
			wmifinfo_led_width, wmifinfo_led_height, BackColor,
			LabelColor, WindGustColor, DataColor, StationTimeColor);
	} else {
		openXwindow(argc, argv, wmifinfo_lcd_xpm, wmifinfo_lcd_bits,
			wmifinfo_lcd_width, wmifinfo_lcd_height, BackColor,
			LabelColor, WindGustColor, DataColor, StationTimeColor);
	}

	// Initialize global variables

	fd = socket(AF_INET, SOCK_DGRAM, IPPROTO_IP);
	ifc.ifc_len = sizeof(struct ifreq) * 10;
	ifc.ifc_buf = malloc(ifc.ifc_len);

	while(1) {
	
		while (XPending(display)) {
			XNextEvent(display, &event);
			
			switch (event.type) {
				case Expose:
					RedrawWindow();
					break;
				case ButtonPress:
					ButtonPressEvent(&event.xbutton);
					break;
				case ButtonRelease:
					break;

			}
		}

		if ((d++ == 5) || (ifno != pifno)) {
		
			d=0;
			
			copyXPMArea(64, 0, 64, 64, 0, 0);
			getifnames();

			if(ifaces>0) {
				ifno = ifno % ifaces;
			
				getifinfo(ifname[ifno], &ifinfo);
			
				if(ifno != pifno) lastbytes = ifinfo.bytes;
				
				sprintf(buf, "%-7s", ifinfo.id);	
				strupper(buf);
				drawtext(buf, &font2, 6, 4);

				if(memcmp(ifinfo.hw, "\x00\x00\x00\x00\x00\x00", 6) != 0) {
					drawhwaddr(ifinfo.hw);
				}
			
				if(ifinfo.ip) drawipaddr(ifinfo.ip, 0);
				if(ifinfo.nm) drawipaddr(ifinfo.nm, 1);
				if(ifinfo.gw) drawipaddr(ifinfo.gw, 2);

				x = ifinfo.sl/4;
				if(x>13) x=13;
				copyXPMArea(4, 82, x*4, 4, 6, 53);
				
				x=0;
				if(ifinfo.state) x += 8;
				if(lastbytes == ifinfo.bytes) x+= 16;
				copyXPMArea(64 + x, 81, 8, 8, 50, 4);
				lastbytes = ifinfo.bytes;
				
			}

			RedrawWindow();
			pifno = ifno;
		}

		usleep(DELAY);
	}

}

void print_usage()
{

	printf(
		"wmifinfo version %s, Ico Doornekamp\n"
		"usage: wmifinfo [-lh] [-i interface]\n"
		"  -i <interface>   Start with given interface, if available\n"
		"  -l               LCD display mode\n"
		"  -h               Print this help\n"
		, VERSION );

}

void parse_cmdline(int argc, char *argv[])
{
	int c;
	
	while((c = getopt(argc, argv, "i:lh")) != EOF) {
		switch(c) {
			case 'i' :
				strncpy(startif, optarg, sizeof(startif));
				break;
			case 'l' :
				mode = MODE_LCD;
				break;
			case 'h' :
				print_usage();
				exit(1);
		}
	}

}


void ButtonPressEvent(XButtonEvent * xev)
{
	char buf[256];
	
	if (xev->type != ButtonPress) return;

	switch (xev->button) {
		case Button1:
		
			ifno++;
			break;
			
		case Button2:
		case Button3:

			if(ifinfo.state == 0) {
				sprintf(buf, "/sbin/ifconfig %s up", ifinfo.id);
				system(buf);
				sprintf(buf, "/sbin/ifup %s", ifinfo.id);
				system(buf);
			} else {
				sprintf(buf, "/sbin/ifconfig %s down", ifinfo.id);
				system(buf);
				sprintf(buf, "/sbin/ifdown %s", ifinfo.id);
				system(buf);
			}
			
			system(buf);

			break;
	}
}


char *strupper(char *str)
{

	int i;

	for (i = 0; i < strlen(str); i++)
		str[i] = toupper(str[i]);

	return str;

}


int getifinfo(char *ifname, struct ifinfo_t *info)
{
	struct ifreq ifr;
	int r;
	struct sockaddr_in *sa;

	static FILE *froute = NULL;
	static FILE *fwireless = NULL;
	static FILE *fdev = NULL;
	
	char parent[16];
	char buf[1024];
	char *p;
	char a[16];
	int b,c,d;

	if(froute == NULL) froute = fopen("/proc/net/route", "r");
	if(fwireless == NULL) fwireless = fopen("/proc/net/wireless", "r");
	if(fdev == NULL) fdev = fopen("/proc/net/dev", "r");

	
	strcpy(parent, ifname);
	p=strchr(parent, ':');
	if(p) *p=0;

	strcpy(info->id, ifname);
	
	strcpy(ifr.ifr_name, ifname);	

	// Get status (UP/DOWN)

	if(ioctl(fd, SIOCGIFFLAGS, &ifr) != -1) {
		sa = (struct sockaddr_in *)&(ifr.ifr_addr);
		info->state = (ifr.ifr_flags & 1) ? 1 : 0;
	} else {
		info->state = 0;
	}
	
	// Get mac address
	
	if(ioctl(fd, SIOCGIFHWADDR, &ifr) != -1) {
		memcpy(info->hw, ifr.ifr_hwaddr.sa_data, 6);
	} else {
		memset(info->hw, 0, 6);
	}
		
	// Get IP address	
	
	if(ioctl(fd, SIOCGIFADDR, &ifr) != -1) {
		sa = (struct sockaddr_in *)&(ifr.ifr_addr);
		info->ip = sa->sin_addr.s_addr;
	} else {
		info->ip = 0;
	}
	
	// Get netmask
	
	if(ioctl(fd, SIOCGIFNETMASK, &ifr) != -1) {
		sa = (struct sockaddr_in *)&(ifr.ifr_addr);
		info->nm = sa->sin_addr.s_addr;
	} else {
		info->nm = 0;
	}
	
	// Get default gateway if on this interface
	
	if(froute != NULL) {
		fseek(froute, 0, 0);
		info->gw = 0;	
		
		while(fgets(buf, sizeof(buf), froute)) {
			r = sscanf(buf, "%s %x %x", a, &b, &c);
				
			if((strcmp(a, info->id) == 0) && (b == 0)) {
				info->gw = c;
			}
		}
		
	}

	// Get wireless link status if wireless
	
	info->sl = 0;
	if(fwireless != NULL) {
		fseek(fwireless, 0, 0);
		
		while(fgets(buf, sizeof(buf), fwireless)) {
			r = sscanf(buf, "%s %d %d ", a, &b, &c);
			if(strchr(a, ':'))  *(strchr(a, ':')) = 0;
			if(strcmp(a, parent) == 0) {
				info->sl = c;
			}
		}
	}
	
	// Get Total tx/rx bytes	

	if(fdev != NULL) {
		fseek(fdev, 0, 0);
		
		while(fgets(buf, sizeof(buf), fdev)) {
			r = sscanf(buf, "%s %d %d %d %d %d %d %d %d %d", a, &b, &d,&d,&d,&d,&d,&d,&d, &c);
			if(strchr(a, ':'))  *(strchr(a, ':')) = 0;
			if(strcmp(a, parent) == 0) {
				info->bytes = b + c;
			}
		}
	}
	
	return(0);
}


void addifname(char *name)
{
	int i;

	if(strcmp(name, "lo") == 0) return;
	
	for(i=0; i<ifaces; i++) {
		if(strcmp(ifname[i], name) == 0) return;
	}

	strcpy(ifname[ifaces], name);
	
	if(strcasecmp(name, startif) == 0) {
		ifno = ifaces;
		startif[0] = 0;
	}
	
	ifaces++;
	
	return;
}


/* 
 * get list of interfaces. First read /proc/net/dev, then do a SIOCGIFCONF
 */
 
void getifnames(void)
{	
	FILE *f;
	char buf[128];
	char *p1, *p2;
	int ifcount;
	int i;
	
	ifaces = 0;
	
	f = fopen("/proc/net/dev", "r");
	
	if(f == NULL) {
		fprintf(stderr, "Can't open /proc/net/dev\n");
		exit(1);
	}

	fgets(buf, sizeof(buf),f);	
	while(fgets(buf, sizeof(buf), f)) {
		p1=buf;
		while(*p1 == ' ') p1++;
		p2=p1;
		while(*p2 && (*p2 != ':')) p2++;
		if(*p2 == ':') {
			*p2=0;
			addifname(p1);
		}
	}
	
	fclose(f);

	ifc.ifc_len = sizeof(struct ifreq) * 10;
	
	if(ioctl(fd, SIOCGIFCONF, &ifc) == -1) {
		fprintf(stderr, "SIOCGIFCONF : Can't get list of interfaces : %s\n", strerror(errno));
		exit(1);
	}
	
	ifcount = ifc.ifc_len / sizeof(struct ifreq);
		
	for(i=0; i<ifcount; i++) {
		addifname(ifc.ifc_req[i].ifr_name);
	}
}

==> wmifinfo-0.03.tgz: wmifinfo-0.03/wmifinfo.c <==
	 
#include <stdio.h>
#include <unistd.h>
#include <getopt.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
#include <X11/X.h>
#include <X11/xpm.h>
#include <arpa/inet.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <net/if.h>
#include <errno.h>
#ifdef ENABLE_NWN_SUPPORT
#include "nwn.h"
#endif

#include "xutils.h"
#include "bitmaps/wmifinfo_led.xbm"
#include "bitmaps/wmifinfo_led.xpm"
#include "bitmaps/wmifinfo_lcd.xbm"
#include "bitmaps/wmifinfo_lcd.xpm"

#define MAXIFS 10
#define DELAY 100000L

char	LabelColor[30]    	= "#79bdbf";
char	WindGustColor[30] 	= "#ff0000";
char	DataColor[30]     	= "#ffbf50";
char	BackColor[30]     	= "#181818";
char	StationTimeColor[30]    = "#c5a6ff";

char startif[16] = "";
char ifname[MAXIFS][16];
int ifaces;
int ifno = 0;
struct ifinfo_t ifinfo;
int fd = 0;
struct ifconf ifc;

struct ifinfo_t {
	char id[16];
	int state;
	char hw[6];
	in_addr_t ip;
	in_addr_t nm;
	in_addr_t gw;
	int sl;
	int bytes;
};

struct font_t {
	char *chars;
	int sx;
	int sy;
	int dx;
	int dy;
	int charspline;
};


#define MODE_LED 1
#define MODE_LCD 2

int mode = MODE_LED;

void parse_cmdline(int argc, char *argv[]);
void ButtonPressEvent(XButtonEvent *);
char *strupper(char *str);
void getifnames(void);
int getifinfo(char *ifname, struct ifinfo_t *info);


struct font_t font1 = { " 0123456789ABCDEF", 64, 74, 4, 5, 17};
struct font_t font2 = { "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789: ", 1, 65, 5, 7, 26};
	

void drawtext(char *str, struct font_t *font, int x, int y)
{
	int i = 0;
	int ix, iy;
	char *p;
		
	while(str[i]) {
		p = strchr(font->chars, str[i]);
		ix = (p) ? (p - font->chars) : 0;
		
		iy = (ix / font->charspline);
		ix = (ix % font->charspline);
	
		copyXPMArea(	font->sx + ix * font->dx, 
				font->sy + iy * font->dy,
				font->dx, 
				font->dy,
				x + font->dx * i, 
				y);
				
		i++;
	}
}


void drawipaddr(in_addr_t addr, int linenum)
{
	char buf[4];
	int i;
	
	for(i=0; i<4; i++) {
		snprintf(buf, 4, "%3d", (addr >> (i*8)) & 255);
		drawtext(buf, &font1, 5 + i*14, 19 + linenum*9);
	}
}

void drawhwaddr(unsigned char *addr)
{
	char buf[4];
	int i;
	
	for(i=0; i<6; i++) {
		snprintf(buf, 4, "%02X", addr[i]);
		drawtext(buf, &font1, 6 + i*9, 46);
	}
}

int main(int argc, char *argv[])
{

	XEvent event;
	char buf[16];
	int d=0;
	int pifno=-1;
	int lastbytes=0;
	int x;
	
	parse_cmdline(argc, argv);

	initXwindow(argc, argv);

	if(mode == MODE_LED) {
		openXwindow(argc, argv, wmifinfo_led_xpm, wmifinfo_led_bits,
			wmifinfo_led_width, wmifinfo_led_height, BackColor,
			LabelColor, WindGustColor, DataColor, StationTimeColor);
	} else {
		openXwindow(argc, argv, wmifinfo_lcd_xpm, wmifinfo_lcd_bits,
			wmifinfo_lcd_width, wmifinfo_lcd_height, BackColor,
			LabelColor, WindGustColor, DataColor, StationTimeColor);
	}

	// Initialize global variables

	fd = socket(AF_INET, SOCK_DGRAM, IPPROTO_IP);
	ifc.ifc_len = sizeof(struct ifreq) * 10;
	ifc.ifc_buf = malloc(ifc.ifc_len);

	while(1) {
	
		while (XPending(display)) {
			XNextEvent(display, &event);
			
			switch (event.type) {
				case Expose:
					RedrawWindow();
					break;
				case ButtonPress:
					ButtonPressEvent(&event.xbutton);
					break;
				case ButtonRelease:
					break;

			}
		}

		if ((d++ == 5) || (ifno != pifno)) {
		
			d=0;
			
			copyXPMArea(64, 0, 64, 64, 0, 0);
			getifnames();

			if(ifaces>0) {
				ifno = ifno % ifaces;
			
				getifinfo(ifname[ifno], &ifinfo);
			
				if(ifno != pifno) lastbytes = ifinfo.bytes;
				
				sprintf(buf, "%-7s", ifinfo.id);	
				strupper(buf);
				drawtext(buf, &font2, 6, 4);

				if(memcmp(ifinfo.hw, "\x00\x00\x00\x00\x00\x00", 6) != 0) {
					drawhwaddr(ifinfo.hw);
				}
			
				if(ifinfo.ip) drawipaddr(ifinfo.ip, 0);
				if(ifinfo.nm) drawipaddr(ifinfo.nm, 1);
				if(ifinfo.gw) drawipaddr(ifinfo.gw, 2);

				x = ifinfo.sl/4;
				if(x>13) x=13;
				copyXPMArea(4, 82, x*4, 4, 6, 53);
				
				x=0;
				if(ifinfo.state) x += 8;
				if(lastbytes == ifinfo.bytes) x+= 16;
				copyXPMArea(64 + x, 81, 8, 8, 50, 4);
				lastbytes = ifinfo.bytes;
				
			}

			RedrawWindow();
			pifno = ifno;
		}

		usleep(DELAY);
	}

}

void print_usage()
{

	printf(
		"wmifinfo version %s, Ico Doornekamp\n"
		"usage: wmifinfo [-lh] [-i interface]\n"
		"  -i <interface>   Start with given interface, if available\n"
		"  -l               LCD display mode\n"
		"  -h               Print this help\n"
		, VERSION );

}

void parse_cmdline(int argc, char *argv[])
{
	int c;
	
	while((c = getopt(argc, argv, "i:lh")) != EOF) {
		switch(c) {
			case 'i' :
				strncpy(startif, optarg, sizeof(startif));
				break;
			case 'l' :
				mode = MODE_LCD;
				break;
			case 'h' :
				print_usage();
				exit(1);
		}
	}

}


void ButtonPressEvent(XButtonEvent * xev)
{
	char buf[256];
	
	if (xev->type != ButtonPress) return;

	switch (xev->button) {
		case Button1:
		
			ifno++;
			break;
			
		case Button2:
		case Button3:

			if(ifinfo.state == 0) {
				sprintf(buf, "/sbin/ifup %s &", ifinfo.id);
				system(buf);
			} else {
				sprintf(buf, "/sbin/ifdown %s &", ifinfo.id);
				system(buf);
			}
			
			system(buf);

			break;
	}
}


char *strupper(char *str)
{

	int i;

	for (i = 0; i < strlen(str); i++)
		str[i] = toupper(str[i]);

	return str;

}


int getifinfo(char *ifname, struct ifinfo_t *info)
{
	struct ifreq ifr;
	int r;
	struct sockaddr_in *sa;

	static FILE *froute = NULL;
	static FILE *fwireless = NULL;
	static FILE *fdev = NULL;
	
	char parent[16];
	char buf[1024];
	char *p;
	char a[16];
	int b,c,d;

	if(froute == NULL) froute = fopen("/proc/net/route", "r");
	if(fwireless == NULL) fwireless = fopen("/proc/net/wireless", "r");
	if(fdev == NULL) fdev = fopen("/proc/net/dev", "r");

	
	strcpy(parent, ifname);
	p=strchr(parent, ':');
	if(p) *p=0;

	strcpy(info->id, ifname);
	
	strcpy(ifr.ifr_name, ifname);	

	// Get status (UP/DOWN)

	if(ioctl(fd, SIOCGIFFLAGS, &ifr) != -1) {
		sa = (struct sockaddr_in *)&(ifr.ifr_addr);
		info->state = (ifr.ifr_flags & 1) ? 1 : 0;
	} else {
		info->state = 0;
	}
	
	// Get mac address
	
	if(ioctl(fd, SIOCGIFHWADDR, &ifr) != -1) {
		memcpy(info->hw, ifr.ifr_hwaddr.sa_data, 6);
	} else {
		memset(info->hw, 0, 6);
	}
		
	// Get IP address	
	
	if(ioctl(fd, SIOCGIFADDR, &ifr) != -1) {
		sa = (struct sockaddr_in *)&(ifr.ifr_addr);
		info->ip = sa->sin_addr.s_addr;
	} else {
		info->ip = 0;
	}
	
	// Get netmask
	
	if(ioctl(fd, SIOCGIFNETMASK, &ifr) != -1) {
		sa = (struct sockaddr_in *)&(ifr.ifr_addr);
		info->nm = sa->sin_addr.s_addr;
	} else {
		info->nm = 0;
	}
	
	// Get default gateway if on this interface
	
	if(froute != NULL) {
		fseek(froute, 0, 0);
		info->gw = 0;	
		
		while(fgets(buf, sizeof(buf), froute)) {
			r = sscanf(buf, "%s %x %x", a, &b, &c);
				
			if((strcmp(a, info->id) == 0) && (b == 0)) {
				info->gw = c;
			}
		}
		
	}

	// Get wireless link status if wireless
	
	info->sl = 0;
	if(fwireless != NULL) {
		fseek(fwireless, 0, 0);
		
		while(fgets(buf, sizeof(buf), fwireless)) {
			r = sscanf(buf, "%s %d %d ", a, &b, &c);
			if(strchr(a, ':'))  *(strchr(a, ':')) = 0;
			if(strcmp(a, parent) == 0) {
				info->sl = c;
			}
		}
	}
	
#ifdef ENABLE_NWN_SUPPORT
	info->sl = nwn_get_link(parent);
#endif
	
	// Get Total tx/rx bytes	

	if(fdev != NULL) {
		fseek(fdev, 0, 0);
		
		while(fgets(buf, sizeof(buf), fdev)) {
			r = sscanf(buf, "%s %d %d %d %d %d %d %d %d %d", a, &b, &d,&d,&d,&d,&d,&d,&d, &c);
			if(strchr(a, ':'))  *(strchr(a, ':')) = 0;
			if(strcmp(a, parent) == 0) {
				info->bytes = b + c;
			}
		}
	}
	
	return(0);
}


void addifname(char *name)
{
	int i;

	if(strcmp(name, "lo") == 0) return;
	
	for(i=0; i<ifaces; i++) {
		if(strcmp(ifname[i], name) == 0) return;
	}

	strcpy(ifname[ifaces], name);
	
	if(strcasecmp(name, startif) == 0) {
		ifno = ifaces;
		startif[0] = 0;
	}
	
	ifaces++;
	
	return;
}


/* 
 * get list of interfaces. First read /proc/net/dev, then do a SIOCGIFCONF
 */
 
void getifnames(void)
{	
	FILE *f;
	char buf[128];
	char *p1, *p2;
	int ifcount;
	int i;
	
	ifaces = 0;
	
	f = fopen("/proc/net/dev", "r");
	
	if(f == NULL) {
		fprintf(stderr, "Can't open /proc/net/dev\n");
		exit(1);
	}

	fgets(buf, sizeof(buf),f);	
	while(fgets(buf, sizeof(buf), f)) {
		p1=buf;
		while(*p1 == ' ') p1++;
		p2=p1;
		while(*p2 && (*p2 != ':')) p2++;
		if(*p2 == ':') {
			*p2=0;
			addifname(p1);
		}
	}
	
	fclose(f);

	ifc.ifc_len = sizeof(struct ifreq) * 10;
	
	if(ioctl(fd, SIOCGIFCONF, &ifc) == -1) {
		fprintf(stderr, "SIOCGIFCONF : Can't get list of interfaces : %s\n", strerror(errno));
		exit(1);
	}
	
	ifcount = ifc.ifc_len / sizeof(struct ifreq);
		
	for(i=0; i<ifcount; i++) {
		addifname(ifc.ifc_req[i].ifr_name);
	}
}
